from __future__ import annotations

"""User context helper that automatically injects the *user_id* into every
Nekuda SDK call.  Returned by :py:meth:`nekuda.NekudaClient.user`.
"""

import uuid
from typing import Any, Dict, Optional, Union

from .client import NekudaClient
from .models import CardDetails, MandateData

__all__ = ["UserContext"]


class UserContext:  # noqa: D101 – docstring below
    """A thin immutable wrapper around :class:`~nekuda.NekudaClient` that
    pins a specific ``user_id``.  This aids readability & reduces repetitive
    parameter passing, especially when an application iterates over many users
    concurrently.

    Instances are *cheap* to create – they only hold two references (client &
    the user_id string) and delegate all network I/O to the shared
    :class:`~nekuda.NekudaClient`.
    """

    __slots__ = ("_client", "user_id")

    def __init__(self, client: NekudaClient, user_id: str) -> None:  # noqa: D401
        if user_id is None:
            raise TypeError("user_id cannot be None")
        if not user_id:
            raise ValueError("user_id cannot be empty")
        self._client = client
        self.user_id = user_id

    # ------------------------------------------------------------------
    # High-level workflows
    # ------------------------------------------------------------------
    def create_mandate(self, mandate: MandateData, *, request_id: Optional[str] = None) -> Dict[str, Any]:
        """Create a mandate for this user.

        Parameters
        ----------
        mandate:
            The :class:`~nekuda.models.MandateData` describing the purchase
            intent.
        request_id:
            Idempotency key.  When *None*, an RFC-4122 UUIDv4 is generated by
            the SDK.
        """
        req_id = request_id or mandate.request_id or str(uuid.uuid4())
        return self._client.create_mandate(
            user_id=self.user_id,
            request_id=req_id,
            mandate_data=mandate.as_dict(),
        )

    # Delegated helpers --------------------------------------------------
    def request_card_reveal_token(self, mandate_id: Union[str, int]) -> Dict[str, str]:
        return self._client.request_card_reveal_token(
            user_id=self.user_id,
            mandate_id=str(mandate_id),
        )

    def reveal_card_details(self, reveal_token: str) -> CardDetails:
        data = self._client.reveal_card_details(
            user_id=self.user_id,
            reveal_token=reveal_token,
        )
        return CardDetails.from_api(data)

    # ------------------------------------------------------------------
    # Misc
    # ------------------------------------------------------------------
    def __repr__(self) -> str:  # noqa: D401 – simple repr
        return f"<UserContext user_id='{self.user_id}'>"
